from h5py import File
from thd_json import projection
from h5schemas.scan_schema import open_schema
from h5schemas.initialize_dataset import initialize_dataset

from h5schemas.detector.add_detector import add_detector_sample
from h5schemas.image.add_image import add_image_sample
from h5schemas.projection_geometry.add_projection_geometry import (
    add_projection_geometry_sample,
)
from h5schemas.header.add_header import check_uuid_time_stamp
from h5schemas.source.add_source import add_source_sample

import numpy as np
from pathlib import Path


def init_projection_dataset(
    h5_file: File,
    image_as_path: bool,
    detector_image_dimensions_px: tuple[int, int],
    pixel_pitch_mm: tuple[float, float],
):
    schema_groups = open_schema(
        projection, "projection.json", image_as_path=image_as_path
    )

    initialize_dataset(h5_file, schema_groups)
    add_detector_sample(
        h5_file["/detector"], detector_image_dimensions_px, pixel_pitch_mm
    )


def add_projection_sample(
    h5_file: File,
    image: Path | np.ndarray,
    detector_center_position_mm: tuple[float, float, float],
    detector_center_orientation_quat: tuple[float, float, float, float],
    focal_spot_position_mm: tuple[float, float, float],
    focal_spot_orientation_quat: tuple[float, float, float, float] | None = None,
    current_ma: float = 0,
    voltage_kv: float = 0,
    uuid: str | None = None,
    timestamp: int | None = None,
):
    uuid, timestamp = check_uuid_time_stamp(uuid, timestamp)
    add_image_sample(h5_file["/image"], image, uuid, timestamp)
    add_projection_geometry_sample(
        h5_file["/projection_geometry"],
        detector_center_position_mm,
        detector_center_orientation_quat,
        focal_spot_position_mm,
        focal_spot_orientation_quat,
        uuid,
        timestamp,
    )
    add_projection_geometry_sample(
        h5_file["/projection_geometry_nominal"],
        detector_center_position_mm,
        detector_center_orientation_quat,
        focal_spot_position_mm,
        focal_spot_orientation_quat,
        uuid,
        timestamp,
    )
    add_source_sample(h5_file["/source"], current_ma, voltage_kv)


if __name__ == "__main__":
    from h5schemas.projection.overwrite_projection import overwrite_projection_at_index

    test_file = File("./examples/projection.h5", "w")
    init_projection_dataset(test_file, False, (128, 128), (0.139, 0.139))

    add_projection_sample(
        test_file,
        1e5 * np.random.random((128, 128)),
        [0.0, 1.0, 2.0],
        [3.0, 4.0, 5.0, 6.0],
        [7.0, 8.0, 9.0],
        [10.0, 11.0, 12.0, 13],
    )
    add_projection_sample(
        test_file,
        1e5 * np.random.random((128, 128)),
        [0.0, 1.0, 2.0],
        [3.0, 4.0, 5.0, 6.0],
        [7.0, 8.0, 9.0],
        [10.0, 11.0, 12.0, 13],
        voltage_kv=100,
    )
    print(test_file["/image/image_array"][...])

    overwrite_projection_at_index(
        test_file,
        0,
        1e5 * np.random.random((128, 128)),
        [0.0, 1.0, 2.0],
        [3.0, 4.0, 5.0, 6.0],
        [7.0, 8.0, 9.0],
        [10.0, 11.0, 12.0, 13],
        current_ma=69.,
        voltage_kv=420.
    )
